#include <windows.h>
#include <plugin.hpp>
#include <mp3tag.h>
#include <mp3tagz.h>
#include <mp3msg.h>
#include <mp3misc.h>
#include <mp3dlg.h>
#include <mp3desc.h>
#include <desc.h>

BOOL ValidPanelState( PanelInfo *lpPanelInfo )
{
     BOOL bResult = FALSE;

     if ( ( lpPanelInfo -> PanelType == PTYPE_FILEPANEL ) &&
          lpPanelInfo -> ItemsNumber )
     {
          if ( PanelItemsSelected( lpPanelInfo ) )
          {
               bResult = TRUE;
          }
          else
          {
               Message(
                    FMSG_WARNING,
                    GetPSI() -> GetMsg( GetPSI() -> ModuleNumber, MError ),
                    GetPSI() -> GetMsg( GetPSI() -> ModuleNumber, MNoItemsSelected )
               );
          }
     }

     return bResult;
}

BOOL UserDialog( tDescMP3Dlg *lpMP3Dlg )
{
     BOOL bResult = FALSE;
     int Selected;

     Selected = GetPSI() -> Dialog(
          GetPSI() -> ModuleNumber,
          -1, -1, 50, 9,
          MP3DESCHELP,
          lpMP3Dlg -> Items(),
          lpMP3Dlg -> ItemsCount()
     );
     if ( ( Selected != lpMP3Dlg -> Cancel() ) && ( Selected != -1 ) )
     {
          if ( lpMP3Dlg -> DataEntered() )
          {
               bResult = TRUE;
          }
          else
          {
               Message(
                    FMSG_WARNING,
                    GetPSI() -> GetMsg( GetPSI() -> ModuleNumber, MError ),
                    GetPSI() -> GetMsg( GetPSI() -> ModuleNumber, MEmptyField )
               );
          }
     }

     return bResult;
}

VOID MacroToDesc(
     LPTSTR szDesc,
     DWORD dwDesc,
     TCHAR Macros,
     const tMP3tagZ *lpTagZ )
{
     TCHAR szSize[MP3SIZEWIDTH];

     switch ( Macros )
     {
          case 'f':
          case 'F':
               if ( NULL == strchr( lpTagZ -> findData. cFileName, ' ' ) )
               {
                    strncat( szDesc, lpTagZ -> findData. cFileName, dwDesc );
               }
               else
               {
                    strncat( szDesc, "\"", dwDesc );
                    strncat( szDesc, lpTagZ -> findData. cFileName, dwDesc - 1 );
                    strncat( szDesc, "\"", dwDesc - 2 );
               }
               break;
          case 't':
          case 'T':
               strncat( szDesc, lpTagZ -> title, dwDesc );
               break;
          case 'a':
          case 'A':
               strncat( szDesc, lpTagZ -> artist, dwDesc );
               break;
          case 'b':
          case 'B':
               strncat( szDesc, lpTagZ -> album, dwDesc );
               break;
          case 'y':
          case 'Y':
               strncat( szDesc, lpTagZ -> year, dwDesc );
               break;
          case 'c':
          case 'C':
               strncat( szDesc, lpTagZ -> comment, dwDesc );
               break;
          case 's':
          case 'S':
               ltoa( lpTagZ -> findData. nFileSizeLow, szSize, SIZE_RADIX );
               strncat( szDesc, szSize, dwDesc );
               break;
          default:
               ;
     }
}

LPTSTR TemplToDesc(
     TCHAR szDesc[],
     DWORD dwDesc,
     LPCTSTR lpcszTempl,
     const tMP3tagZ *lpTagZ )
{
     BOOL MacroOn = FALSE;

     ZeroMemory( szDesc, dwDesc );
     for ( DWORD idx = 0, idxD = 0; ( idxD < dwDesc - 1 ) && lpcszTempl[idx]; idx++ )
     {
          if ( lpcszTempl[idx] == MACRO )
          {
               if ( MacroOn )
               {
                    szDesc[idxD] = MACRO;
                    idxD++;
                    MacroOn = FALSE;
               }
               else
               {
                    MacroOn = TRUE;
               }
               continue;
          }
          if ( MacroOn )
          {
               MacroToDesc(
                    szDesc,
                    dwDesc - 1 - idxD,
                    lpcszTempl[idx],
                    lpTagZ
               );
               idxD = strlen( szDesc );
               MacroOn = FALSE;
          }
          else
          {
               szDesc[idxD] = lpcszTempl[idx];
               idxD++;
          }
     }

     return szDesc;
}

int Warning( LPCTSTR lpcszFileName )
{
     TCHAR szBuffer[MAX_PATH];

     strcpy( szBuffer, GetPSI() -> GetMsg( GetPSI() -> ModuleNumber, MDescExist ) );
     strcat( szBuffer, lpcszFileName );

     const char *Items[] =
     {
          GetPSI() -> GetMsg( GetPSI() -> ModuleNumber, MWarning ),
          szBuffer,
          "\1",
          GetPSI() -> GetMsg( GetPSI() -> ModuleNumber, MUpdate ),
          GetPSI() -> GetMsg( GetPSI() -> ModuleNumber, MUpdateAll ),
          GetPSI() -> GetMsg( GetPSI() -> ModuleNumber, MLeave ),
          GetPSI() -> GetMsg( GetPSI() -> ModuleNumber, MLeaveAll ),
          GetPSI() -> GetMsg( GetPSI() -> ModuleNumber, MCancel )
     };

     return GetPSI() -> Message(
          GetPSI() -> ModuleNumber,
          FMSG_WARNING,
          NULL,
          ( char** ) Items,
          sizeof( Items ) / sizeof( Items[0] ),
          5
     );
}

BOOL UpdateMP3Desc(
     DESC lpDesc[],
     DWORD dwDesc,
     DWORD *lpdwcDesc,
     PanelInfo *lpPanelInfo,
     LPCTSTR lpcszTemplate,
     LPCTSTR lpcszTemplateNoTag )
{
     BOOL bResult = TRUE;
     tMP3tagZ tagZ;
     DWORD dwFindDesc;
     DWORD dwIndex;
     int iUserSelect = 0;
     TCHAR szBuffer[MAX_DESC];

     for ( DWORD idx = 0; ( idx < lpPanelInfo -> ItemsNumber ) &&
          ( *lpdwcDesc < dwDesc ); idx++ )
     {
          if ( !( lpPanelInfo -> PanelItems[idx]. Flags & PPIF_SELECTED ) )
          {
               continue;
          }
          if ( !ValidMP3Ext( lpPanelInfo -> PanelItems[idx]. FindData. cFileName ) )
          {
               continue;
          }
          if ( FindDesc(
               lpPanelInfo -> PanelItems[idx]. FindData. cFileName,
               lpDesc,
               *lpdwcDesc,
               &dwFindDesc ) )
          {
               if ( iUserSelect == 3 ) // LeaveAll
               {
                    continue;
               }
               if ( iUserSelect != 1 ) // UpdateAll
               {
                    iUserSelect = Warning( lpPanelInfo ->
                         PanelItems[idx]. FindData. cFileName );

                    if ( iUserSelect == 2 ) // Leave
                    {
                         continue;
                    }

                    if ( ( iUserSelect == -1 ) || ( iUserSelect == 4 ) ) // Esc, Cancel
                    {
                         break;
                    }
               }
               dwIndex = dwFindDesc;
          }
          else
          {
               dwIndex = *lpdwcDesc;
               ( *lpdwcDesc )++;
          }
          if ( !ReadMP3TagZ(
               lpPanelInfo -> PanelItems[idx]. FindData. cFileName,
               &tagZ ) )
          {
               Message(
                    FMSG_ERRORTYPE | FMSG_WARNING,
                    lpPanelInfo -> PanelItems[idx]. FindData. cFileName,
                    GetPSI() -> GetMsg( GetPSI() -> ModuleNumber, MMP3ReadError )
               );
               bResult = FALSE;
               break;
          }
          TemplToDesc(
               szBuffer,
               sizeof( szBuffer ),
               tagZ. exist ? lpcszTemplate : lpcszTemplateNoTag,
               &tagZ
          );
          if ( !AddDesc(
               lpDesc,
               dwDesc,
               tagZ. findData. cFileName,
               szBuffer,
               dwIndex ) )
          {
               bResult = FALSE;
               break;
          }
          lpPanelInfo -> PanelItems[idx]. Flags &= ~PPIF_SELECTED;
     }

     return bResult;
}

BOOL DescMP3()
{
     BOOL bResult = FALSE;
     PanelInfo panelInfo;
     tDescMP3Dlg MP3Dlg;
     DESC *lpDesc;
     DWORD dwcDesc;
     DWORD dwDesc;

     if ( !GetPSI() -> Control( INVALID_HANDLE_VALUE, FCTL_GETPANELINFO, &panelInfo ) )
     {
          return FALSE;
     }

     if ( !ValidPanelState( &panelInfo ) )
     {
          return FALSE;
     }

     if ( !UserDialog( &MP3Dlg ) )
     {
          return FALSE;
     }

     lpDesc = ( DESC* ) HeapAlloc(
          GetProcessHeap(),
          HEAP_ZERO_MEMORY,
          sizeof( lpDesc[0] ) * MAX_DESC_LINES
     );

     if ( lpDesc == NULL )
     {
          return FALSE;
     }

     dwDesc = HeapSize( GetProcessHeap(), 0, lpDesc ) / sizeof( lpDesc[0] );

     bResult = LoadDescFile(
          MP3Dlg. FileName(),
          lpDesc,
          dwDesc,
          &dwcDesc
     );

     if ( bResult )
     {
          bResult = UpdateMP3Desc(
               lpDesc,
               dwDesc,
               &dwcDesc,
               &panelInfo,
               MP3Dlg. Template(),
               MP3Dlg. TemplateNoTag()
          );
     }

     if ( bResult )
     {
          bResult = SaveDescFile(
               MP3Dlg. FileName(),
               lpDesc,
               dwcDesc
          );
     }

     if ( !bResult )
     {
          Message(
               FMSG_ERRORTYPE | FMSG_WARNING,
               GetPSI() -> GetMsg( GetPSI() -> ModuleNumber, MError ),
               GetPSI() -> GetMsg( GetPSI() -> ModuleNumber, MDescFileError )
          );
     }

     HeapFree( GetProcessHeap(), 0, lpDesc );

     GetPSI() -> Control( INVALID_HANDLE_VALUE, FCTL_SETSELECTION, &panelInfo );
     GetPSI() -> Control( INVALID_HANDLE_VALUE, FCTL_REDRAWPANEL, NULL );

     return bResult;
}

