#include <windows.h>
#include <plugin.hpp>
#include <mp3tag.h>
#include <mp3tagz.h>
#include <mp3msg.h>
#include <mp3misc.h>
#include <mp3reg.h>
#include <mp3dlg.h>
#include <mp3edit.h>

BOOL UserDialog(
     LPCTSTR lpcszCaption,
     tMP3tagZ *lpTagZ,
     BOOL *lpDelete,
     BOOL *lpWrite )
{
     tEditMP3Dlg MP3Dlg;
     DWORD dwFlags;
     int Button;

     dwFlags = GetPluginFlags();

     MP3Dlg. SetCaption( lpcszCaption );
     MP3Dlg. ReadMP3tagZ( lpTagZ, dwFlags & FL_FILL_EMPTY );

     Button = GetPSI() -> Dialog(
          GetPSI() -> ModuleNumber,
          -1, -1, MP3EDITDLGWIDTH, 12,
          MP3EDITHELP,
          MP3Dlg. Items(),
          MP3Dlg. ItemsCount()
     );

     if ( ( Button == MP3Dlg. Cancel() ) || ( Button == -1 ) )
     {
          return FALSE;
     }

     if ( Button == MP3Dlg. Untag() )
     {
          *lpWrite = lpTagZ -> exist;
          *lpDelete = TRUE;
          return TRUE;
     }

     *lpDelete = FALSE;

     if ( !MP3Dlg. ValidDate() )
     {
          Message(
               FMSG_WARNING,
               GetPSI() -> GetMsg( GetPSI() -> ModuleNumber, MError ),
               GetPSI() -> GetMsg( GetPSI() -> ModuleNumber, MInvalidDate )
          );
          return FALSE;
     }

     if ( !MP3Dlg. ValidStyle() )
     {
          Message(
               FMSG_ERRORTYPE | FMSG_WARNING,
               GetPSI() -> GetMsg( GetPSI() -> ModuleNumber, MError ),
               GetPSI() -> GetMsg( GetPSI() -> ModuleNumber, MInvalidStyle )
          );
          return FALSE;
     }

     if ( Button == MP3Dlg. Ok() )
     {
          MP3Dlg. WriteMP3tagZ( lpTagZ );
          *lpWrite = TRUE;
     }
     else
     {
          *lpWrite = FALSE;
     }

     return TRUE;
}

BOOL GetUserDialog( tMP3tagZ *lpTagZ, BOOL *lpDelete, BOOL *lpWrite )
{
     TCHAR Caption[MAX_PATH];

     strcpy( Caption, lpTagZ -> findData. cFileName );
     if ( !( lpTagZ -> exist ) )
     {
          strcat(
               Caption,
               GetPSI() -> GetMsg( GetPSI() -> ModuleNumber,
               MNoTag )
          );
     }

     size_t caplen = strlen( Caption );

     if ( caplen > MP3EDITDLGCAPWIDTH )
     {
          strcpy( Caption, Caption + ( caplen - MP3EDITDLGCAPWIDTH ) );
     }

     return UserDialog( Caption, lpTagZ, lpDelete, lpWrite );
}

BOOL EditMP3( LPCTSTR MP3FileName, BOOL *lpWrote )
{
     BOOL bResult = TRUE ;
     tMP3tagZ tagZ;
     BOOL Write;
     BOOL Delete;

     if ( !ValidMP3Ext( MP3FileName ) )
     {
          Message(
               FMSG_WARNING,
               MP3FileName,
               GetPSI() -> GetMsg( GetPSI() -> ModuleNumber, MNotMP3Ext )
          );
          bResult = FALSE;
     }

     if ( bResult )
     {
          if ( !ReadMP3TagZ( MP3FileName, &tagZ ) )
          {
               Message(
                    FMSG_ERRORTYPE | FMSG_WARNING,
                    MP3FileName,
                    GetPSI() -> GetMsg( GetPSI() -> ModuleNumber, MMP3ReadError )
               );
               bResult = FALSE;
          }
     }

     if ( bResult )
     {
          bResult = GetUserDialog( &tagZ, &Delete, &Write );
     }

     if ( bResult )
     {
          if ( lpWrote != NULL )
          {
               *lpWrote = Write;
          }

          if ( Write )
          {
               if  ( Delete )
               {
                    bResult = DeleteMP3Tag( tagZ. findData. cFileName );
               }
               else
               {
                    bResult = WriteMP3TagZ( &tagZ );
               }
               if ( !bResult )
               {
                    Message(
                         FMSG_ERRORTYPE | FMSG_WARNING,
                         tagZ. findData. cFileName,
                         GetPSI() -> GetMsg( GetPSI() -> ModuleNumber, MMP3WriteError )
                    );
               }
          }
     }

     return bResult;
}

BOOL EditMP3( PanelInfo *lpPanelInfo )
{
     BOOL bResult = TRUE;
     BOOL bWrote;

     for ( int idx = 0, stop = lpPanelInfo -> ItemsNumber; idx < stop; idx++ )
     {
          if ( !( lpPanelInfo -> PanelItems[idx]. Flags & PPIF_SELECTED ) )
          {
               continue;
          }
          if ( !EditMP3(
               lpPanelInfo -> PanelItems[idx]. FindData. cFileName,
               &bWrote ) )
          {
               bResult = FALSE;
               break;
          }
          if ( bWrote )
          {
               lpPanelInfo -> PanelItems[idx]. Flags &= ~PPIF_SELECTED;
          }
     }

     return bResult;
}

BOOL EditMP3()
{
     PanelInfo panelInfo;

     if ( !GetPSI() -> Control( INVALID_HANDLE_VALUE, FCTL_GETPANELINFO, &panelInfo ) )
     {
          return FALSE;
     }

     if ( panelInfo. PanelType != PTYPE_FILEPANEL )
     {
          return FALSE;
     }

     if ( !panelInfo. ItemsNumber )
     {
          return FALSE;
     }

     BOOL bResult;

     if ( PanelItemsSelected( &panelInfo ) )
     {
          bResult = EditMP3( &panelInfo );
          GetPSI() -> Control( INVALID_HANDLE_VALUE, FCTL_SETSELECTION, &panelInfo );
          GetPSI() -> Control( INVALID_HANDLE_VALUE, FCTL_REDRAWPANEL, NULL );
     }
     else
     {
          bResult = EditMP3(
               panelInfo. PanelItems[panelInfo. CurrentItem]. FindData. cFileName,
               NULL
          );
     }

     return bResult;
}

