#include <windows.h>
#include <plugin.hpp>
#include <mp3tagz.h>
#include <mp3misc.h>
#include <stdlib.h>

static TCHAR MP3TagSign[] = { 'T', 'A', 'G' };

// ==========================================================================
tMP3tag *ZTAGtoTAG( tMP3tag *dest, const tMP3tagZ *src )
{
    ZeroMemory( dest, sizeof(*dest) );
    CopyMemory( dest->tag, MP3TagSign, sizeof( dest -> tag ) );
    CopyMemory( dest->title, src->title, strlen( src -> title ) );
    CopyMemory( dest->artist, src->artist, strlen( src -> artist ) );
    CopyMemory( dest->album, src->album, strlen( src -> album ) );
    CopyMemory( dest->year, src->year, strlen( src -> year ) );
    CopyMemory( dest->comment, src->comment, strlen( src -> comment ) );
    dest -> style = src -> style;

    return dest;
}

// ==========================================================================
tMP3tagZ *TAGtoZTAG( tMP3tagZ *dest, const tMP3tag *src )
{
     ZeroTagZ( dest );
     strncpyz( dest->title, src->title, sizeof( dest -> title ) );
     strncpyz( dest->artist, src->artist, sizeof( dest -> artist ) );
     strncpyz( dest->album, src->album, sizeof( dest -> album ) );
     strncpyz( dest->year, src->year, sizeof( dest -> year ) );
     strncpyz( dest->comment, src->comment, sizeof( dest -> comment ) );
     dest -> style = src -> style;

     return dest;
}

// ==========================================================================
tMP3tagZ *TrimZTAG( tMP3tagZ *lpTagZ )
{
     strncpyz( lpTagZ->title, lpTagZ->title, sizeof( lpTagZ -> title ) );
     strncpyz( lpTagZ->artist, lpTagZ->artist, sizeof( lpTagZ -> artist ) );
     strncpyz( lpTagZ->album, lpTagZ->album, sizeof( lpTagZ -> album ) );
     strncpyz( lpTagZ->year, lpTagZ->year, sizeof( lpTagZ -> year ) );
     strncpyz( lpTagZ->comment, lpTagZ->comment, sizeof( lpTagZ -> comment ) );

     return lpTagZ;
}

// ==========================================================================
BOOL MP3tag( const tMP3tag *tag )
{
     return !strncmp( tag -> tag, MP3TagSign, sizeof( tag -> tag ) );
}

BOOL MP3tagZ( const tMP3tagZ *tagZ )
{
     return
          tagZ -> title[0] ||
          tagZ -> artist[0] ||
          tagZ -> album[0] ||
          tagZ -> year[0] ||
          tagZ -> comment[0];
}

// ==========================================================================
BOOL ReadMP3Tag( LPCTSTR Mp3FileName, tMP3tag *tag )
{
     tMP3tag __tag;
     HANDLE hMp3File;
     DWORD __dw;

     hMp3File = CreateFile(
          Mp3FileName,
          GENERIC_READ,
          FILE_SHARE_READ | FILE_SHARE_WRITE,
          NULL,
          OPEN_EXISTING,
          FILE_ATTRIBUTE_NORMAL,
          NULL
     );

     if ( hMp3File == INVALID_HANDLE_VALUE )
     {
          return FALSE;
     }

     if ( -1 == SetFilePointer(
          hMp3File,
          -sizeof( tMP3tag ),
          NULL,
          FILE_END ) )
     {
          CloseHandle( hMp3File );
          ZeroMemory( tag, sizeof( *tag ) );
          return TRUE;
     }

     if ( !ReadFile(
          hMp3File,
          ( LPVOID ) &__tag,
          sizeof( __tag ),
          &__dw,
          NULL ) )
     {
          CloseHandle( hMp3File );
          return FALSE;
     }

     CloseHandle( hMp3File );

     if ( !MP3tag( &__tag ) )
     {
          ZeroMemory( tag, sizeof( *tag ) );
     }
     else
     {
          *tag = __tag;
     }

     return TRUE;
}

// ==========================================================================
BOOL WriteMP3Tag( LPCTSTR Mp3FileName, const tMP3tag *tag )
{
     tMP3tag __tag;
     HANDLE hMp3File;
     DWORD __dw;
     LONG lMoveTo;

     hMp3File = CreateFile(
          Mp3FileName,
          GENERIC_READ | GENERIC_WRITE,
          FILE_SHARE_READ | FILE_SHARE_WRITE,
          NULL,
          OPEN_EXISTING,
          FILE_ATTRIBUTE_NORMAL,
          NULL
     );

     if ( hMp3File == INVALID_HANDLE_VALUE )
     {
          return FALSE;
     }

     SetFilePointer(
          hMp3File,
          -sizeof( tMP3tag ),
          NULL,
          FILE_END
     );

     if ( !ReadFile(
          hMp3File,
          ( LPVOID ) &__tag,
          sizeof( __tag ),
          &__dw,
          NULL ) )
     {
          CloseHandle( hMp3File );
          return FALSE;
     }

     if ( !MP3tag( &__tag ) )
     {
          lMoveTo = 0;
     }
     else
     {
          lMoveTo = -sizeof( tMP3tag );
     }

     SetFilePointer(
          hMp3File,
          lMoveTo,
          NULL,
          FILE_END
     );

     if ( !WriteFile(
          hMp3File,
          ( LPVOID ) tag,
          sizeof( *tag ),
          &__dw,
          NULL ) )
     {
          CloseHandle( hMp3File );
          return FALSE;
     }

     CloseHandle( hMp3File );

     return TRUE;
}

// ==========================================================================
BOOL DeleteMP3Tag( LPCTSTR lpcszMP3 )
{
     tMP3tag __tag;
     HANDLE hMP3;
     DWORD __dw = 0;

     hMP3 = CreateFile(
          lpcszMP3,
          GENERIC_READ | GENERIC_WRITE,
          FILE_SHARE_READ | FILE_SHARE_WRITE,
          NULL,
          OPEN_EXISTING,
          FILE_ATTRIBUTE_NORMAL,
          NULL
     );

     if ( hMP3 == INVALID_HANDLE_VALUE )
     {
          return FALSE;
     }

     if ( -1 == SetFilePointer(
          hMP3,
          -sizeof( tMP3tag ),
          NULL,
          FILE_END ) )
     {
          CloseHandle( hMP3 );
          return FALSE;
     }

     if ( !ReadFile(
          hMP3,
          ( LPVOID ) &__tag,
          sizeof( __tag ),
          &__dw,
          NULL ) )
     {
          CloseHandle( hMP3 );
          return FALSE;
     }

     if ( !MP3tag( &__tag ) )
     {
          CloseHandle( hMP3 );
          return TRUE;
     }

     if ( -1 == SetFilePointer(
          hMP3,
          -sizeof( tMP3tag ),
          NULL,
          FILE_END ) )
     {
          CloseHandle( hMP3 );
          return FALSE;
     }

     if ( !SetEndOfFile( hMP3 ) )
     {
          CloseHandle( hMP3 );
          return FALSE;
     }

     CloseHandle( hMP3 );

     return TRUE;
}

// ==========================================================================
tMP3tag *CharToOemTag( tMP3tag *tag )
{
    CharToOemBuff( tag->title,    tag->title,   sizeof( tag->title )   );
    CharToOemBuff( tag->artist,   tag->artist,  sizeof( tag->artist )  );
    CharToOemBuff( tag->album,    tag->album,   sizeof( tag->album )   );
    CharToOemBuff( tag->year,     tag->year,    sizeof( tag->year )    );
    CharToOemBuff( tag->comment,  tag->comment, sizeof( tag->comment ) );

    return tag;
}

// ==========================================================================
tMP3tag *OemToCharTag( tMP3tag *tag )
{
    OemToCharBuff( tag->title,    tag->title,   sizeof( tag->title )   );
    OemToCharBuff( tag->artist,   tag->artist,  sizeof( tag->artist )  );
    OemToCharBuff( tag->album,    tag->album,   sizeof( tag->album )   );
    OemToCharBuff( tag->year,     tag->year,    sizeof( tag->year )    );
    OemToCharBuff( tag->comment,  tag->comment, sizeof( tag->comment ) );

    return tag;
}

// ==========================================================================
BOOL ReadMP3TagZ( LPCTSTR MP3name, tMP3tagZ *lpTagZ )
{
     HANDLE hFile = FindFirstFile(
          MP3name,
          &( lpTagZ -> findData )
     );

     if ( hFile == INVALID_HANDLE_VALUE )
     {
          return FALSE;
     }

     FindClose( hFile );

     tMP3tag tag;

     if ( !ReadMP3Tag( lpTagZ -> findData. cFileName, &tag ) )
     {
          return FALSE;
     }

     CharToOemTag( &tag );
     lpTagZ -> exist = MP3tag( &tag );
     TAGtoZTAG( lpTagZ, &tag );

     return TRUE;

}

// ==========================================================================
BOOL WriteMP3TagZ( tMP3tagZ *lpTagZ )
{
     if ( !MP3tagZ( lpTagZ ) )
     {
          return TRUE;
     }

     tMP3tag tag;

     TrimZTAG( lpTagZ );
     ZTAGtoTAG( &tag, lpTagZ );
     OemToCharTag( &tag );

     return WriteMP3Tag( lpTagZ -> findData. cFileName, &tag );
}

// ==========================================================================
tMP3tagZ *ZeroTagZ( tMP3tagZ *lpTagZ )
{
     ZeroMemory( lpTagZ -> title, sizeof( lpTagZ -> title ) );
     ZeroMemory( lpTagZ -> artist, sizeof( lpTagZ -> artist ) );
     ZeroMemory( lpTagZ -> album, sizeof( lpTagZ -> album ) );
     ZeroMemory( lpTagZ -> year, sizeof( lpTagZ -> year ) );
     ZeroMemory( lpTagZ -> comment, sizeof( lpTagZ -> comment ) );
     ZeroMemory( &( lpTagZ -> style ), sizeof( lpTagZ -> style ) );

     return lpTagZ;
}

